/*-------------------------------
   CMOS.C -- Displays CMOS
			(c) Tim Clarke, 1992
  -------------------------------*/

#include <windows.h>
#include <dos.h>
#include <string.h>

#define NUMLINES 4
#define NUMCOLS 42

long FAR PASCAL WndProc (HWND, WORD, WORD, LONG) ;

BYTE GetCmosByte (BYTE) ;

BYTE CmosBuffer [64] ;

int PASCAL WinMain (HANDLE hInstance, HANDLE hPrevInstance, LPSTR lpszCmdLine, int nCmdShow)
	{
	static char szAppName [] = "Cmos" ;
	HWND hwnd ;
	MSG msg ;
	WNDCLASS wndclass ;

	if (!hPrevInstance)
		{
		wndclass.style			= CS_HREDRAW | CS_VREDRAW ;
		wndclass.lpfnWndProc	= WndProc ;
		wndclass.cbClsExtra		= 0 ;
		wndclass.cbWndExtra		= 0 ;
		wndclass.hInstance		= hInstance ;
		wndclass.hIcon			= LoadIcon (NULL, IDI_APPLICATION) ;
		wndclass.hCursor		= LoadCursor (NULL, IDC_ARROW) ;
		wndclass.hbrBackground	= COLOR_WINDOW + 1 ;
		wndclass.lpszMenuName	= NULL ;
		wndclass.lpszClassName	= szAppName ;

		RegisterClass (&wndclass) ;
		}

	hwnd = CreateWindow (szAppName, "CMOS",
		WS_OVERLAPPEDWINDOW,
		CW_USEDEFAULT, CW_USEDEFAULT,
		CW_USEDEFAULT, CW_USEDEFAULT,
		NULL, NULL, hInstance, NULL) ;

	ShowWindow (hwnd, nCmdShow) ;
	UpdateWindow (hwnd) ;

	while (GetMessage (&msg, NULL, 0, 0))
		{
		TranslateMessage (&msg) ;
		DispatchMessage (&msg) ;
		}
	return msg.wParam ;
	}

long FAR PASCAL WndProc (HWND hwnd, WORD message, WORD wParam, LONG lParam)
	{
	static	short 	cxChar, cyChar, cxClient, cyClient,
					nVscrollPos, nVscrollMax, nHscrollPos, nHscrollMax ;
	static	char	sBuffer[NUMCOLS] ;
	HDC 			hdc ;
	PAINTSTRUCT 	ps ;
	TEXTMETRIC 		tm ;
	short 			i, j, x, y ;
	short			nLine, nFirstLine, nLastLine ;
	short			nVscrollInc, nHscrollInc ;
	unsigned		nBuffOff, nByteIndex ;
	switch (message)
		{
		case WM_CREATE :
			for (i = 0; i < 2048; i++)
				CmosBuffer[i] = GetCmosByte (i);

			hdc = GetDC (hwnd) ;

			SelectObject (hdc, GetStockObject (SYSTEM_FIXED_FONT)) ;

			GetTextMetrics (hdc, &tm) ;

			cxChar = tm.tmAveCharWidth ;
			cyChar = tm.tmHeight + tm.tmExternalLeading ;

			ReleaseDC (hwnd, hdc) ;

			return 0 ;

		case WM_SIZE :
			cyClient = HIWORD (lParam) ;
			cxClient = LOWORD (lParam) ;

			nVscrollMax = max (0, (NUMLINES + 1) - (cyClient / cyChar) ) ;
			nVscrollPos = min (nVscrollPos, nVscrollMax) ;

			SetScrollRange (hwnd, SB_VERT, 0, nVscrollMax, FALSE) ;
			SetScrollPos (hwnd, SB_VERT, nVscrollPos, TRUE) ;

			nHscrollMax = max (0, (NUMCOLS + 1) - (cxClient / cxChar) ) ;
			nHscrollPos = min (nHscrollPos, nHscrollMax) ;

			SetScrollRange (hwnd, SB_HORZ, 0, nHscrollMax, FALSE) ;
			SetScrollPos (hwnd, SB_HORZ, nHscrollPos, TRUE) ;
			return 0 ;

		case WM_VSCROLL :
			switch (wParam)
				{
				case SB_TOP :
					nVscrollInc = -nVscrollPos ;
					break ;

				case SB_BOTTOM :
					nVscrollInc = nVscrollMax - nVscrollPos ;
					break ;

				case SB_LINEUP :
					nVscrollInc = -1 ;
					break ;

				case SB_LINEDOWN :
					nVscrollInc = 1 ;
					break ;

				case SB_PAGEUP :
					nVscrollInc = min (-1, -cyClient / cyChar) ;
					break ;

				case SB_PAGEDOWN :
					nVscrollInc = max (1, cyClient / cyChar) ;
					break ;

				case SB_THUMBTRACK :
					nVscrollInc = LOWORD (lParam) - nVscrollPos ;
					break ;

				default :
					nVscrollInc = 0 ;
				}
			if ((nVscrollInc = max (-nVscrollPos, min (nVscrollInc,
				nVscrollMax - nVscrollPos))) != 0)
				{
				nVscrollPos += nVscrollInc ;
				ScrollWindow (hwnd, 0, -cyChar * nVscrollInc, NULL, NULL) ;
				SetScrollPos (hwnd, SB_VERT, nVscrollPos, TRUE) ;
				UpdateWindow (hwnd) ;
				}
			return 0 ;

		case WM_HSCROLL :
			switch (wParam)
				{
				case SB_LINEUP :
					nHscrollInc = -1 ;
					break ;

				case SB_LINEDOWN :
					nHscrollInc = 1 ;
					break ;

				case SB_PAGEUP :
					nHscrollInc = -8 ;
					break ;

				case SB_PAGEDOWN :
					nHscrollInc = 8 ;
					break ;

				case SB_THUMBPOSITION :
					nHscrollInc = LOWORD (lParam) - nHscrollPos ;
					break ;

				default :
					nHscrollInc = 0 ;
				}
			if ((nHscrollInc = max (-nHscrollPos, min (nHscrollInc,
				nHscrollMax - nHscrollPos))) != 0)
				{
				nHscrollPos += nHscrollInc ;
				ScrollWindow (hwnd, -cxChar * nHscrollInc, 0, NULL, NULL) ;
				SetScrollPos (hwnd, SB_HORZ, nHscrollPos, TRUE) ;
				}
			return 0 ;

		case WM_KEYDOWN :
			switch (wParam)
				{
				case VK_HOME :
					SendMessage (hwnd, WM_VSCROLL, SB_TOP, 0L) ;
					break ;

				case VK_END :
					SendMessage (hwnd, WM_VSCROLL, SB_BOTTOM, 0L) ;
					break ;

				case VK_PRIOR :
					SendMessage (hwnd, WM_VSCROLL, SB_PAGEUP, 0L) ;
					break ;

				case VK_NEXT :
					SendMessage (hwnd, WM_VSCROLL, SB_PAGEDOWN, 0L) ;
					break ;

				case VK_UP :
					SendMessage (hwnd, WM_VSCROLL, SB_LINEUP, 0L) ;
					break ;

				case VK_DOWN :
					SendMessage (hwnd, WM_VSCROLL, SB_LINEDOWN, 0L) ;
					break ;

				case VK_LEFT :
					SendMessage (hwnd, WM_HSCROLL, SB_LINEUP, 0L) ;
					break ;

				case VK_RIGHT :
					SendMessage (hwnd, WM_HSCROLL, SB_LINEDOWN, 0L) ;
					break ;

				}
			return 0 ;

		case WM_PAINT :
			hdc = BeginPaint ( hwnd, &ps) ;

			SelectObject (hdc, GetStockObject (SYSTEM_FIXED_FONT)) ;

			nFirstLine = max (0, nVscrollPos + (ps.rcPaint.top / cyChar) - 1) ;
			nLastLine = min (NUMLINES, nVscrollPos + (ps.rcPaint.bottom / cyChar) + 1) ;

			for ( nLine = nFirstLine ; nLine < nLastLine ; nLine++)
				{
				x = cxChar * (1 - nHscrollPos) ;
				y = cyChar * (1 - nVscrollPos + nLine) ;
				wsprintf (sBuffer, "%04X: ", (nLine * 16)) ;
				for ( i = 0 ; i < 4 ; i++)
					{
					for ( j = 0 ; j < 4 ; j++)
						{
						nBuffOff = 6 + (j * 2) + (i * 9);
						nByteIndex = j + (i * 4) + (nLine * 16) ;
						wsprintf (sBuffer + nBuffOff, "%02X", CmosBuffer[nByteIndex]) ;
						}
					nBuffOff = 14 + (i * 9) ;
					sBuffer[nBuffOff] = ' ' ;
					}
				TextOut (hdc, x, y, sBuffer, NUMCOLS) ;
				}

			EndPaint (hwnd, &ps) ;
			return 0 ;

		case WM_DESTROY :
			PostQuitMessage (0) ;
			return 0 ;
		}

	return DefWindowProc (hwnd, message, wParam, lParam) ;
	}

BYTE GetCmosByte (BYTE address)
	{
	BYTE byte;
	__emit__ (0x9C) ;
	disable () ;
	outportb (0x70, (address | 0x80)) ;
	__emit__ (0xEB, 0x00) ;
	byte = inportb (0x71);
	outportb (0x70, 0x0D) ;
	__emit__ (0xEB, 0x00) ;
	inportb (0x71) ;
	__emit__ (0xEB, 0x00) ;
	enable () ;
	__emit__ (0x9D) ;
	return byte;
	}

